@@ -1,11 +1,13 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
 // Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002, 2003
+// Copyright Dirk Lemstra 2014-2017
 //
 // Implementation of Drawable (Graphic objects)
 //
 
 #define MAGICKCORE_IMPLEMENTATION  1
+#define MAGICK_PLUSPLUS_IMPLEMENTATION 1
 #define MAGICK_DRAWABLE_IMPLEMENTATION
 
 #include "Magick++/Include.h"
@@ -17,117 +19,106 @@
 
 using namespace std;
 
-MagickDLLDecl int Magick::operator == ( const Magick::Coordinate& left_,
-                                        const Magick::Coordinate& right_ )
+MagickPPExport int Magick::operator == (const Magick::Coordinate& left_,
+  const Magick::Coordinate& right_)
 {
-  return ( ( left_.x() == right_.x() ) && ( left_.y() == right_.y() ) );
+  return((left_.x() == right_.x()) && (left_.y() == right_.y()));
 }
-MagickDLLDecl int Magick::operator != ( const Magick::Coordinate& left_,
-                                        const Magick::Coordinate& right_ )
+
+MagickPPExport int Magick::operator != (const Magick::Coordinate& left_,
+  const Magick::Coordinate& right_)
 {
-  return ( ! (left_ == right_) );
+  return(!(left_ == right_));
 }
-MagickDLLDecl int Magick::operator >  ( const Magick::Coordinate& left_,
-                                        const Magick::Coordinate& right_ )
+
+MagickPPExport int Magick::operator > (const Magick::Coordinate& left_,
+  const Magick::Coordinate& right_)
 {
-  return ( !( left_ < right_ ) && ( left_ != right_ ) );
+  return (!(left_ < right_) && (left_ != right_));
 }
-MagickDLLDecl int Magick::operator <  ( const Magick::Coordinate& left_,
-                                        const Magick::Coordinate& right_ )
+
+MagickPPExport int Magick::operator < (const Magick::Coordinate& left_,
+  const Magick::Coordinate& right_)
 {
   // Based on distance from origin
-  return  ( (sqrt(left_.x()*left_.x() + left_.y()*left_.y())) <
-            (sqrt(right_.x()*right_.x() + right_.y()*right_.y())) );
-}
-MagickDLLDecl int Magick::operator >= ( const Magick::Coordinate& left_,
-                                        const Magick::Coordinate& right_ )
-{
-  return ( ( left_ > right_ ) || ( left_ == right_ ) );
-}
-MagickDLLDecl int Magick::operator <= ( const Magick::Coordinate& left_,
-                                        const Magick::Coordinate& right_ )
-{
-  return ( ( left_ < right_ ) || ( left_ == right_ ) );
+  return((sqrt(left_.x()*left_.x() + left_.y()*left_.y())) <
+    (sqrt(right_.x()*right_.x() + right_.y()*right_.y())));
 }
 
-/*virtual*/
-Magick::DrawableBase::~DrawableBase ( void )
+MagickPPExport int Magick::operator >= (const Magick::Coordinate& left_,
+  const Magick::Coordinate& right_)
+{
+  return((left_ > right_) || (left_ == right_));
+}
+
+MagickPPExport int Magick::operator <= (const Magick::Coordinate& left_,
+  const Magick::Coordinate& right_)
+{
+  return((left_ < right_) || (left_ == right_));
+}
+
+/* DrawableBase */
+Magick::DrawableBase::DrawableBase()
 {
 }
 
-// Constructor
-Magick::Drawable::Drawable ( void )
-  : dp(0)
+Magick::DrawableBase::~DrawableBase(void)
 {
 }
 
-// Construct from DrawableBase
-Magick::Drawable::Drawable ( const Magick::DrawableBase& original_ )
+void Magick::DrawableBase::operator()(MagickCore::DrawingWand * context_) const
+{
+  (void) context_;
+}
+
+Magick::DrawableBase* Magick::DrawableBase::copy() const
+{
+  return new DrawableBase(*this);
+}
+
+/* Drawable */
+Magick::Drawable::Drawable(void)
+  : dp((Magick::DrawableBase *) NULL)
+{
+}
+
+Magick::Drawable::Drawable(const Magick::DrawableBase& original_)
   : dp(original_.copy())
 {
 }
 
-// Destructor
-Magick::Drawable::~Drawable ( void )
+Magick::Drawable::~Drawable(void)
 {
   delete dp;
-  dp = 0;
+  dp=(Magick::DrawableBase *) NULL;
 }
 
-// Copy constructor
-Magick::Drawable::Drawable ( const Magick::Drawable& original_ )
-  : dp(original_.dp? original_.dp->copy(): 0)
+Magick::Drawable::Drawable(const Magick::Drawable& original_)
+  : dp((original_.dp != (Magick::DrawableBase *) NULL ? original_.dp->copy() :
+    (Magick::DrawableBase *) NULL))
 {
 }
 
-// Assignment operator
-Magick::Drawable& Magick::Drawable::operator= (const Magick::Drawable& original_ )
+Magick::Drawable& Magick::Drawable::operator= (
+  const Magick::Drawable& original_)
 {
+  DrawableBase
+    *temp_dp;
+
   if (this != &original_)
     {
-      DrawableBase* temp_dp = (original_.dp ? original_.dp->copy() : 0);
+      temp_dp=(original_.dp != (Magick::DrawableBase *) NULL ?
+        original_.dp->copy() : (Magick::DrawableBase *) NULL);
       delete dp;
-      dp = temp_dp;
+      dp=temp_dp;
     }
-  return *this;
+  return(*this);
 }
 
-// Operator to invoke contained object
-void Magick::Drawable::operator()( MagickCore::DrawingWand * context_ ) const
+void Magick::Drawable::operator()(MagickCore::DrawingWand * context_) const
 {
-  if(dp)
-    dp->operator()( context_ );
-}
-
-MagickDLLDecl int Magick::operator == ( const Magick::Drawable& /*left_*/,
-                                        const Magick::Drawable& /*right_*/ )
-{
-  return ( 1 );
-}
-MagickDLLDecl int Magick::operator != ( const Magick::Drawable& /*left_*/,
-                                        const Magick::Drawable& /*right_*/ )
-{
-  return ( 0 );
-}
-MagickDLLDecl int Magick::operator > ( const Magick::Drawable& /*left_*/,
-                                       const Magick::Drawable& /*right_*/ )
-{
-  return ( 0 );
-}
-MagickDLLDecl int Magick::operator <  ( const Magick::Drawable& /*left_*/,
-                                        const Magick::Drawable& /*right_*/ )
-{
-  return  ( 0 );
-}
-MagickDLLDecl int Magick::operator >= ( const Magick::Drawable& left_,
-                                        const Magick::Drawable& right_ )
-{
-  return ( ( left_ > right_ ) || ( left_ == right_ ) );
-}
-MagickDLLDecl int Magick::operator <= ( const Magick::Drawable& left_,
-                                        const Magick::Drawable& right_ )
-{
-  return ( ( left_ < right_ ) || ( left_ == right_ ) );
+  if (dp != (Magick::DrawableBase *) NULL)
+    dp->operator()(context_);
 }
 
 /*virtual*/
@@ -179,37 +170,6 @@ void Magick::VPath::operator()( MagickCore::DrawingWand * context_ ) const
     dp->operator()( context_ );
 }
 
-MagickDLLDecl int Magick::operator == ( const Magick::VPath& /*left_*/,
-                                        const Magick::VPath& /*right_*/ )
-{
-  return ( 1 );
-}
-MagickDLLDecl int Magick::operator != ( const Magick::VPath& /*left_*/,
-                                        const Magick::VPath& /*right_*/ )
-{
-  return ( 0 );
-}
-MagickDLLDecl int Magick::operator > ( const Magick::VPath& /*left_*/,
-                                       const Magick::VPath& /*right_*/ )
-{
-  return ( 0 );
-}
-MagickDLLDecl int Magick::operator <  ( const Magick::VPath& /*left_*/,
-                                        const Magick::VPath& /*right_*/ )
-{
-  return  ( 0 );
-}
-MagickDLLDecl int Magick::operator >= ( const Magick::VPath& left_,
-                                        const Magick::VPath& right_ )
-{
-  return ( ( left_ > right_ ) || ( left_ == right_ ) );
-}
-MagickDLLDecl int Magick::operator <= ( const Magick::VPath& left_,
-                                        const Magick::VPath& right_ )
-{
-  return ( ( left_ < right_ ) || ( left_ == right_ ) );
-}
-
 //
 // Drawable Objects
 //
@@ -242,6 +202,20 @@ Magick::DrawableBase* Magick::DrawableAffine::copy() const
   return new DrawableAffine(*this);
 }
 
+Magick::DrawableAlpha::~DrawableAlpha(void)
+{
+}
+
+void Magick::DrawableAlpha::operator()(MagickCore::DrawingWand * context_) const
+{
+  DrawAlpha(context_,_x,_y,_paintMethod);
+}
+
+Magick::DrawableBase* Magick::DrawableAlpha::copy() const
+{
+  return new DrawableAlpha(*this);
+}
+
 // Arc
 Magick::DrawableArc::~DrawableArc( void )
 {
@@ -275,7 +249,7 @@ Magick::DrawableBezier::~DrawableBezier( void )
 }
 void Magick::DrawableBezier::operator()( MagickCore::DrawingWand * context_ ) const
 {
-  unsigned long num_coords = (unsigned long) _coordinates.size();
+  size_t num_coords = (size_t) _coordinates.size();
   PointInfo *coordinates = new PointInfo[num_coords];
 
   PointInfo *q = coordinates;
@@ -297,6 +271,120 @@ Magick::DrawableBase* Magick::DrawableBezier::copy() const
   return new DrawableBezier(*this);
 }
 
+
+/* DrawableBorderColor */
+Magick::DrawableBorderColor::DrawableBorderColor(const Magick::Color &color_)
+  : _color(color_)
+{
+}
+
+Magick::DrawableBorderColor::DrawableBorderColor
+  (const Magick::DrawableBorderColor &original_)
+  : DrawableBase(original_),
+    _color(original_._color)
+{
+}
+
+Magick::DrawableBorderColor::~DrawableBorderColor(void)
+{
+}
+
+void Magick::DrawableBorderColor::operator()(
+  MagickCore::DrawingWand *context_) const
+{
+  PixelInfo
+    color;
+
+  PixelWand
+    *pixel_wand;
+
+  color=static_cast<PixelInfo>(_color);
+  pixel_wand=NewPixelWand();
+  PixelSetPixelColor(pixel_wand,&color);
+  DrawSetBorderColor(context_,pixel_wand);
+  pixel_wand=DestroyPixelWand(pixel_wand);
+}
+
+void Magick::DrawableBorderColor::color(const Color &color_)
+{
+  _color=color_;
+}
+
+Magick::Color Magick::DrawableBorderColor::color(void) const
+{
+  return(_color);
+}
+
+Magick::DrawableBase* Magick::DrawableBorderColor::copy() const
+{
+  return(new DrawableBorderColor(*this));
+}
+
+
+/* DrawableClipRule */
+Magick::DrawableClipRule::DrawableClipRule(const FillRule fillRule_)
+{
+  _fillRule=fillRule_;
+}
+
+Magick::DrawableClipRule::~DrawableClipRule(void)
+{
+}
+
+void Magick::DrawableClipRule::operator()(
+  MagickCore::DrawingWand * context_) const
+{
+  DrawSetClipRule(context_,_fillRule);
+}
+
+void Magick::DrawableClipRule::fillRule(const FillRule fillRule_)
+{
+  _fillRule=fillRule_;
+}
+
+Magick::FillRule Magick::DrawableClipRule::fillRule(void) const
+{
+  return(_fillRule);
+}
+
+Magick::DrawableBase* Magick::DrawableClipRule::copy() const
+{
+  return(new DrawableClipRule(*this));
+}
+
+
+/* DrawableClipUnits */
+Magick::DrawableClipUnits::DrawableClipUnits(const ClipPathUnits units_)
+{
+  _units = units_;
+}
+
+Magick::DrawableClipUnits::~DrawableClipUnits(void)
+{
+}
+
+void Magick::DrawableClipUnits::operator()(
+  MagickCore::DrawingWand * context_) const
+{
+  DrawSetClipUnits(context_, _units);
+}
+
+void Magick::DrawableClipUnits::units(const ClipPathUnits units_)
+{
+  _units = units_;
+}
+
+Magick::ClipPathUnits Magick::DrawableClipUnits::units(void) const
+{
+  return(_units);
+}
+
+Magick::DrawableBase* Magick::DrawableClipUnits::copy() const
+{
+  return(new DrawableClipUnits(*this));
+}
+
+
 //
 //Clip Path 
 //
@@ -550,6 +638,31 @@ Magick::DrawableBase* Magick::DrawableCompositeImage::copy() const
   return new DrawableCompositeImage(*this);
 }
 
+Magick::DrawableDensity::DrawableDensity(const Point &density_)
+  : _density(density_)
+{
+}
+
+Magick::DrawableDensity::DrawableDensity(const std::string &density_)
+  : _density(density_)
+{
+}
+
+Magick::DrawableDensity::~DrawableDensity(void)
+{
+}
+
+void Magick::DrawableDensity::operator()(
+  MagickCore::DrawingWand *context_) const
+{
+  DrawSetDensity(context_,_density.c_str());
+}
+
+Magick::DrawableBase* Magick::DrawableDensity::copy() const
+{
+  return(new DrawableDensity(*this));
+}
+
 // Ellipse
 Magick::DrawableEllipse::~DrawableEllipse( void )
 {
@@ -582,9 +695,9 @@ Magick::DrawableFillColor::~DrawableFillColor( void )
 void Magick::DrawableFillColor::operator()
   ( MagickCore::DrawingWand * context_ ) const
 {
-  PixelPacket color = static_cast<PixelPacket>(_color);
+  PixelInfo color = static_cast<PixelInfo>(_color);
   PixelWand *pixel_wand=NewPixelWand();
-  PixelSetQuantumColor(pixel_wand,&color);
+  PixelSetPixelColor(pixel_wand,&color);
   DrawSetFillColor(context_,pixel_wand);
   pixel_wand=DestroyPixelWand(pixel_wand);
 }
@@ -593,6 +706,44 @@ Magick::DrawableBase* Magick::DrawableFillColor::copy() const
   return new DrawableFillColor(*this);
 }
 
+/* DrawableFillPatternUrl */
+Magick::DrawableFillPatternUrl::DrawableFillPatternUrl(const std::string &url_)
+  : _url(url_)
+{
+}
+
+Magick::DrawableFillPatternUrl::DrawableFillPatternUrl(
+  const Magick::DrawableFillPatternUrl& original_)
+  : DrawableBase(original_),
+  _url(original_._url)
+{
+}
+
+Magick::DrawableFillPatternUrl::~DrawableFillPatternUrl(void)
+{
+}
+
+void Magick::DrawableFillPatternUrl::operator()(
+  MagickCore::DrawingWand * context_) const
+{
+  DrawSetFillPatternURL(context_, _url.c_str());
+}
+
+void Magick::DrawableFillPatternUrl::url(const std::string &url_)
+{
+  _url = url_;
+}
+
+std::string Magick::DrawableFillPatternUrl::url(void) const
+{
+  return(_url);
+}
+
+Magick::DrawableBase* Magick::DrawableFillPatternUrl::copy() const
+{
+  return(new DrawableFillPatternUrl(*this));
+}
+
 // Specify drawing fill fule
 Magick::DrawableFillRule::~DrawableFillRule ( void )
 {
@@ -607,15 +758,16 @@ Magick::DrawableBase* Magick::DrawableFillRule::copy() const
   return new DrawableFillRule(*this);
 }
 
-// Specify drawing fill opacity
-Magick::DrawableFillOpacity::~DrawableFillOpacity ( void )
+Magick::DrawableFillOpacity::~DrawableFillOpacity(void)
 {
 }
+
 void Magick::DrawableFillOpacity::operator()
-  ( MagickCore::DrawingWand * context_ ) const
+  (MagickCore::DrawingWand *context_) const
 {
-  DrawSetFillOpacity( context_, _opacity );
+  DrawSetFillOpacity(context_,_opacity);
 }
+
 Magick::DrawableBase* Magick::DrawableFillOpacity::copy() const
 {
   return new DrawableFillOpacity(*this);
@@ -632,7 +784,7 @@ Magick::DrawableFont::DrawableFont ( const std::string &font_ )
 }
 Magick::DrawableFont::DrawableFont ( const std::string &family_,
                                      Magick::StyleType style_,
-                                     const unsigned long weight_,
+                                     const unsigned int weight_,
                                      Magick::StretchType stretch_ )
   : _font(),
     _family(family_),
@@ -708,19 +860,6 @@ Magick::DrawableBase* Magick::DrawableLine::copy() const
   return new DrawableLine(*this);
 }
 
-// Change pixel matte value to transparent using PaintMethod
-Magick::DrawableMatte::~DrawableMatte ( void )
-{
-}
-void Magick::DrawableMatte::operator()( MagickCore::DrawingWand * context_ ) const
-{
-  DrawMatte( context_, _x, _y, _paintMethod );
-}
-Magick::DrawableBase* Magick::DrawableMatte::copy() const
-{
-  return new DrawableMatte(*this);
-}
-
 // Drawable Path
 Magick::DrawablePath::DrawablePath ( const VPathList &path_ )
   : _path(path_)
@@ -793,7 +932,7 @@ Magick::DrawablePolygon::~DrawablePolygon ( void )
 void Magick::DrawablePolygon::operator()
   ( MagickCore::DrawingWand * context_ ) const
 {
-  unsigned long num_coords = (unsigned long) _coordinates.size();
+  size_t num_coords = (size_t) _coordinates.size();
   PointInfo *coordinates = new PointInfo[num_coords];
 
   PointInfo *q = coordinates;
@@ -833,7 +972,7 @@ Magick::DrawablePolyline::~DrawablePolyline ( void )
 void Magick::DrawablePolyline::operator()
   ( MagickCore::DrawingWand * context_ ) const
 {
-  unsigned long num_coords = (unsigned long) _coordinates.size();
+  size_t num_coords = (size_t) _coordinates.size();
   PointInfo *coordinates = new PointInfo[num_coords];
 
   PointInfo *q = coordinates;
@@ -899,8 +1038,8 @@ Magick::DrawableBase* Magick::DrawablePopPattern::copy() const
 
 // Push Pattern definition
 Magick::DrawablePushPattern::DrawablePushPattern
-( const std::string &id_, long x_, long y_,
-  long width_, long height_ )
+( const std::string &id_, ssize_t x_, ssize_t y_,
+  size_t width_, size_t height_ )
   : _id(id_),
     _x(x_),
     _y(y_),
@@ -967,8 +1106,8 @@ Magick::DrawableRoundRectangle::~DrawableRoundRectangle ( void )
 void Magick::DrawableRoundRectangle::operator()
   ( MagickCore::DrawingWand * context_ ) const
 {
-  DrawRoundRectangle( context_, _centerX,_centerY, _width,_hight,
-                      _cornerWidth, _cornerHeight);
+  DrawRoundRectangle(context_,_upperLeftX,_upperLeftY,_lowerRightX,
+    _lowerRightY,_cornerWidth, _cornerHeight);
 }
 Magick::DrawableBase* Magick::DrawableRoundRectangle::copy() const
 {
@@ -1016,122 +1155,128 @@ Magick::DrawableBase* Magick::DrawableSkewY::copy() const
   return new DrawableSkewY(*this);
 }
 
-// Stroke dasharray
-Magick::DrawableDashArray::DrawableDashArray( const double* dasharray_ )
+/* DrawableStrokeDashArray */
+Magick::DrawableStrokeDashArray::DrawableStrokeDashArray(const double* dasharray_)
   : _size(0),
     _dasharray(0)
 {
-  dasharray( dasharray_ );
+  dasharray(dasharray_);
 }
-// Deprecated, do not use for new code, and migrate existing code to
-// using double*
-Magick::DrawableDashArray::DrawableDashArray( const unsigned int* dasharray_ )
-  : _size(0),
-    _dasharray(0)
-{
-  dasharray( dasharray_ );
-}
-Magick::DrawableDashArray::DrawableDashArray
-(const Magick::DrawableDashArray& original_)
+
+Magick::DrawableStrokeDashArray::DrawableStrokeDashArray(
+  const Magick::DrawableStrokeDashArray& original_)
   : DrawableBase (original_),
-    _size(0),
-    _dasharray(0)
+    _size(original_._size),
+    _dasharray(new double[_size+1])
 {
-  dasharray( original_._dasharray );
+  // Copy elements
+  {
+    for (size_t i=0; i < _size; i++)
+      _dasharray[i]=original_._dasharray[i];
+    _dasharray[_size]=0.0;
+  }
 }
-Magick::DrawableDashArray::~DrawableDashArray( void )
-{
-  delete _dasharray;
-  _size = 0;
-  _dasharray = 0;
-}
-Magick::DrawableDashArray& Magick::DrawableDashArray::operator=
-(const Magick::DrawableDashArray &original_)
-{
-  if( this != &original_ )
-    {
-      dasharray( original_._dasharray );
-    }
-  return *this;
-}
-void Magick::DrawableDashArray::operator()
-  ( MagickCore::DrawingWand * context_ ) const
-{
-  (void) DrawSetStrokeDashArray( context_, _size, _dasharray );
-}
-Magick::DrawableBase* Magick::DrawableDashArray::copy() const
-{
-  return new DrawableDashArray(*this);
-}
-void Magick::DrawableDashArray::dasharray ( const double* dasharray_ )
-{
-  _dasharray=(double *) RelinquishMagickMemory(_dasharray);
 
-  if(dasharray_)
-    {
-      // Count elements in dash array
-      unsigned int n = 0;
-      {
-        const double *p = dasharray_;
-        while(*p++ != 0)
-          n++;
-      }
-      _size = n;
+Magick::DrawableStrokeDashArray::~DrawableStrokeDashArray(void)
+{
+  delete [] _dasharray;
+  _size=0;
+  _dasharray=(double *) NULL;
+}
 
-      // Allocate elements
-      _dasharray=static_cast<double*>(AcquireMagickMemory((n+1)*sizeof(double)));
+Magick::DrawableStrokeDashArray& Magick::DrawableStrokeDashArray::operator=(
+  const Magick::DrawableStrokeDashArray &original_)
+{
+  if (this != &original_)
+    {
+      delete [] _dasharray;
+      _size=original_._size;
+      _dasharray = new double[_size+1];
       // Copy elements
       {
-        double *q = _dasharray;
-        const double *p = dasharray_;
-        while( *p )
-          *q++=*p++;
-        *q=0;
+        for (size_t i=0; i < _size; i++)
+          _dasharray[i]=original_._dasharray[i];
+        _dasharray[_size]=0.0;
       }
     }
+  return(*this);
 }
-// This method is deprecated.  Don't use for new code, and migrate existing
-// code to the const double* version.
-void Magick::DrawableDashArray::dasharray( const unsigned int* dasharray_ )
-{
-  _dasharray=(double *) RelinquishMagickMemory(_dasharray);
 
-  if(dasharray_)
+void Magick::DrawableStrokeDashArray::operator()(
+  MagickCore::DrawingWand *context_) const
+{
+  (void) DrawSetStrokeDashArray(context_,(const unsigned long) _size,
+    _dasharray);
+}
+
+Magick::DrawableBase *Magick::DrawableStrokeDashArray::copy() const
+{
+  return(new DrawableStrokeDashArray(*this));
+}
+
+void Magick::DrawableStrokeDashArray::dasharray(const double* dasharray_)
+{
+  size_t
+    n;
+
+  delete [] _dasharray;
+  _size=0;
+  _dasharray=0;
+
+  if (dasharray_ != (const double *) NULL)
     {
+      const double
+        *p;
+
       // Count elements in dash array
-      unsigned int n = 0;
+      n=0;
       {
-        const unsigned int *p = dasharray_;
-        while(*p++ != 0)
+        p = dasharray_;
+        while(*p++ != 0.0)
           n++;
       }
-      _size = n;
+      _size=n;
 
       // Allocate elements
-      _dasharray=static_cast<double*>(AcquireMagickMemory((n+1)*sizeof(double)));
+      _dasharray=new double[_size+1];
       // Copy elements
       {
-        double *q = _dasharray;
-        const unsigned int *p = dasharray_;
-        while( *p )
-          *q++=static_cast<double>(*p++);
-        *q=0;
+        for (size_t i=0; i < _size; i++)
+          _dasharray[i]=dasharray_[i];
+        _dasharray[_size]=0.0;
       }
     }
 }
 
-// Stroke dashoffset
-Magick::DrawableDashOffset::~DrawableDashOffset ( void )
+const double* Magick::DrawableStrokeDashArray::dasharray(void) const
+{
+  return(_dasharray);
+}
+
+/* DrawableStrokeDashOffset */
+Magick::DrawableStrokeDashOffset::~DrawableStrokeDashOffset(void)
 {
 }
-void Magick::DrawableDashOffset::operator()
-  ( MagickCore::DrawingWand * context_ ) const
+
+void Magick::DrawableStrokeDashOffset::operator()
+  ( MagickCore::DrawingWand * context_) const
 {
-  DrawSetStrokeDashOffset( context_, _offset );
+  DrawSetStrokeDashOffset(context_,_offset);
 }
-Magick::DrawableBase* Magick::DrawableDashOffset::copy() const
+
+Magick::DrawableBase* Magick::DrawableStrokeDashOffset::copy() const
 {
-  return new DrawableDashOffset(*this);
+  return(new DrawableStrokeDashOffset(*this));
+}
+
+void Magick::DrawableStrokeDashOffset::offset(const double offset_)
+{
+  _offset=offset_;
+}
+
+double Magick::DrawableStrokeDashOffset::offset(void) const
+{
+  return(_offset);
 }
 
 // Stroke linecap
@@ -1176,6 +1321,46 @@ Magick::DrawableBase* Magick::DrawableMiterLimit::copy() const
   return new DrawableMiterLimit(*this);
 }
 
+
+/* DrawableStrokePatternUrl */
+Magick::DrawableStrokePatternUrl::DrawableStrokePatternUrl(
+  const std::string &url_)
+  : _url(url_)
+{
+}
+
+Magick::DrawableStrokePatternUrl::DrawableStrokePatternUrl(
+  const Magick::DrawableStrokePatternUrl& original_)
+  : DrawableBase(original_),
+  _url(original_._url)
+{
+}
+
+Magick::DrawableStrokePatternUrl::~DrawableStrokePatternUrl(void)
+{
+}
+
+void Magick::DrawableStrokePatternUrl::operator()(
+  MagickCore::DrawingWand * context_) const
+{
+  DrawSetStrokePatternURL(context_, _url.c_str());
+}
+
+void Magick::DrawableStrokePatternUrl::url(const std::string &url_)
+{
+  _url = url_;
+}
+
+std::string Magick::DrawableStrokePatternUrl::url(void) const
+{
+  return(_url);
+}
+
+Magick::DrawableBase* Magick::DrawableStrokePatternUrl::copy() const
+{
+  return(new DrawableStrokePatternUrl(*this));
+}
+
 // Stroke antialias
 Magick::DrawableStrokeAntialias::~DrawableStrokeAntialias ( void )
 {
@@ -1209,9 +1394,9 @@ Magick::DrawableStrokeColor::~DrawableStrokeColor ( void )
 void Magick::DrawableStrokeColor::operator()
   ( MagickCore::DrawingWand * context_ ) const
 {
-  PixelPacket color = static_cast<PixelPacket>(_color);
+  PixelInfo color = static_cast<PixelInfo>(_color);
   PixelWand *pixel_wand=NewPixelWand();
-  PixelSetQuantumColor(pixel_wand,&color);
+  PixelSetPixelColor(pixel_wand,&color);
   DrawSetStrokeColor(context_,pixel_wand);
   pixel_wand=DestroyPixelWand(pixel_wand);
 }
@@ -1220,15 +1405,16 @@ Magick::DrawableBase* Magick::DrawableStrokeColor::copy() const
   return new DrawableStrokeColor(*this);
 }
 
-// Stroke opacity
-Magick::DrawableStrokeOpacity::~DrawableStrokeOpacity ( void )
+Magick::DrawableStrokeOpacity::~DrawableStrokeOpacity(void)
 {
 }
+
 void Magick::DrawableStrokeOpacity::operator()
-  ( MagickCore::DrawingWand * context_ ) const
+  (MagickCore::DrawingWand * context_) const
 {
-  DrawSetStrokeOpacity( context_, _opacity );
+  DrawSetStrokeOpacity(context_,_opacity);
 }
+
 Magick::DrawableBase* Magick::DrawableStrokeOpacity::copy() const
 {
   return new DrawableStrokeOpacity(*this);
@@ -1288,6 +1474,45 @@ Magick::DrawableBase* Magick::DrawableText::copy() const
   return new DrawableText(*this);
 }
 
+/* DrawableTextAlignment */
+Magick::DrawableTextAlignment::DrawableTextAlignment(
+  Magick::AlignType alignment_)
+  : _alignment(alignment_)
+{
+}
+
+Magick::DrawableTextAlignment::DrawableTextAlignment
+(const Magick::DrawableTextAlignment &original_)
+  : DrawableBase(original_),
+  _alignment(original_._alignment)
+{
+}
+
+Magick::DrawableTextAlignment::~DrawableTextAlignment(void)
+{
+}
+
+void Magick::DrawableTextAlignment::operator()(
+  MagickCore::DrawingWand * context_) const
+{
+  DrawSetTextAlignment(context_, _alignment);
+}
+
+void Magick::DrawableTextAlignment::alignment(AlignType alignment_)
+{
+  _alignment=alignment_;
+}
+
+Magick::AlignType Magick::DrawableTextAlignment::alignment(void) const
+{
+  return(_alignment);
+}
+
+Magick::DrawableBase* Magick::DrawableTextAlignment::copy() const
+{
+  return new DrawableTextAlignment(*this);
+}
+
 // Text antialias
 Magick::DrawableTextAntialias::DrawableTextAntialias ( bool flag_ )
   : _flag(flag_)
@@ -1312,6 +1537,7 @@ Magick::DrawableBase* Magick::DrawableTextAntialias::copy() const
   return new DrawableTextAntialias(*this);
 }
 
+
 // Decoration (text decoration)
 Magick::DrawableTextDecoration::DrawableTextDecoration
   ( Magick::DecorationType decoration_ )
@@ -1337,6 +1563,134 @@ Magick::DrawableBase* Magick::DrawableTextDecoration::copy() const
   return new DrawableTextDecoration(*this);
 }
 
+// DrawableTextDirection
+Magick::DrawableTextDirection::DrawableTextDirection(
+  DirectionType direction_)
+  : _direction(direction_)
+{
+}
+
+Magick::DrawableTextDirection::~DrawableTextDirection(void)
+{
+}
+
+void Magick::DrawableTextDirection::operator()(
+  MagickCore::DrawingWand *context_) const
+{
+  DrawSetTextDirection(context_,_direction);
+}
+
+void Magick::DrawableTextDirection::direction(DirectionType direction_)
+{
+  _direction=direction_;
+}
+
+Magick::DirectionType Magick::DrawableTextDirection::direction(void) const
+{
+  return(_direction);
+}
+
+Magick::DrawableBase *Magick::DrawableTextDirection::copy() const
+{
+  return new DrawableTextDirection(*this);
+}
+
+// DrawableTextInterlineSpacing
+Magick::DrawableTextInterlineSpacing::DrawableTextInterlineSpacing(
+  double spacing_)
+  : _spacing(spacing_)
+{
+}
+
+Magick::DrawableTextInterlineSpacing::~DrawableTextInterlineSpacing(void)
+{
+}
+
+void Magick::DrawableTextInterlineSpacing::operator()(
+  MagickCore::DrawingWand *context_) const
+{
+  DrawSetTextInterlineSpacing(context_,_spacing);
+}
+
+void Magick::DrawableTextInterlineSpacing::spacing(double spacing_)
+{
+  _spacing=spacing_;
+}
+
+double Magick::DrawableTextInterlineSpacing::spacing(void) const
+{
+  return(_spacing);
+}
+
+Magick::DrawableBase *Magick::DrawableTextInterlineSpacing::copy() const
+{
+  return new DrawableTextInterlineSpacing(*this);
+}
+
+// DrawableTextInterwordSpacing
+Magick::DrawableTextInterwordSpacing::DrawableTextInterwordSpacing(
+  double spacing_)
+  : _spacing(spacing_)
+{
+}
+
+Magick::DrawableTextInterwordSpacing::~DrawableTextInterwordSpacing(void)
+{
+}
+
+void Magick::DrawableTextInterwordSpacing::operator()(
+  MagickCore::DrawingWand *context_) const
+{
+  DrawSetTextInterwordSpacing(context_,_spacing);
+}
+
+void Magick::DrawableTextInterwordSpacing::spacing(double spacing_)
+{
+  _spacing=spacing_;
+}
+
+double Magick::DrawableTextInterwordSpacing::spacing(void) const
+{
+  return(_spacing);
+}
+
+Magick::DrawableBase *Magick::DrawableTextInterwordSpacing::copy() const
+{
+  return new DrawableTextInterwordSpacing(*this);
+}
+
+// DrawableTextKerning
+Magick::DrawableTextKerning::DrawableTextKerning(
+  double kerning_)
+  : _kerning(kerning_)
+{
+}
+
+Magick::DrawableTextKerning::~DrawableTextKerning(void)
+{
+}
+
+void Magick::DrawableTextKerning::operator()(
+  MagickCore::DrawingWand *context_) const
+{
+  DrawSetTextKerning(context_,_kerning);
+}
+
+void Magick::DrawableTextKerning::kerning(double kerning_)
+{
+  _kerning=kerning_;
+}
+
+double Magick::DrawableTextKerning::kerning(void) const
+{
+  return(_kerning);
+}
+
+Magick::DrawableBase *Magick::DrawableTextKerning::copy() const
+{
+  return new DrawableTextKerning(*this);
+}
+
 // Set text undercolor
 Magick::DrawableTextUnderColor::DrawableTextUnderColor
 ( const Magick::Color &color_ )
@@ -1355,9 +1709,9 @@ Magick::DrawableTextUnderColor::~DrawableTextUnderColor ( void )
 void Magick::DrawableTextUnderColor::operator()
   ( MagickCore::DrawingWand * context_ ) const
 {
-  PixelPacket color = static_cast<PixelPacket>(_color);
+  PixelInfo color = static_cast<PixelInfo>(_color);
   PixelWand *pixel_wand=NewPixelWand();
-  PixelSetQuantumColor(pixel_wand,&color);
+  PixelSetPixelColor(pixel_wand,&color);
   DrawSetTextUnderColor(context_,pixel_wand);
   pixel_wand=DestroyPixelWand(pixel_wand);
 }
@@ -1401,32 +1755,32 @@ Magick::DrawableBase* Magick::DrawableViewbox::copy() const
 //
 // PathArcArgs
 //
-MagickDLLDecl int Magick::operator == ( const Magick::PathArcArgs& /*left_*/,
+MagickPPExport int Magick::operator == ( const Magick::PathArcArgs& /*left_*/,
                                         const Magick::PathArcArgs& /*right_*/ )
 {
   return ( 1 );
 }
-MagickDLLDecl int Magick::operator != ( const Magick::PathArcArgs& /*left_*/,
+MagickPPExport int Magick::operator != ( const Magick::PathArcArgs& /*left_*/,
                                         const Magick::PathArcArgs& /*right_*/ )
 {
   return ( 0 );
 }
-MagickDLLDecl int Magick::operator > ( const Magick::PathArcArgs& /*left_*/,
+MagickPPExport int Magick::operator > ( const Magick::PathArcArgs& /*left_*/,
                                        const Magick::PathArcArgs& /*right_*/ )
 {
   return ( 0 );
 }
-MagickDLLDecl int Magick::operator <  ( const Magick::PathArcArgs& /*left_*/,
+MagickPPExport int Magick::operator <  ( const Magick::PathArcArgs& /*left_*/,
                                         const Magick::PathArcArgs& /*right_*/ )
 {
   return  ( false );
 }
-MagickDLLDecl int Magick::operator >= ( const Magick::PathArcArgs& left_,
+MagickPPExport int Magick::operator >= ( const Magick::PathArcArgs& left_,
                                         const Magick::PathArcArgs& right_ )
 {
   return ( ( left_ > right_ ) || ( left_ == right_ ) );
 }
-MagickDLLDecl int Magick::operator <= ( const Magick::PathArcArgs& left_,
+MagickPPExport int Magick::operator <= ( const Magick::PathArcArgs& left_,
                                         const Magick::PathArcArgs& right_ )
 {
   return ( ( left_ < right_ ) || ( left_ == right_ ) );
@@ -1552,32 +1906,32 @@ Magick::VPathBase* Magick::PathClosePath::copy() const
 //
 // Path Curveto (Cubic Bezier)
 //
-MagickDLLDecl int Magick::operator == ( const Magick::PathCurvetoArgs& /*left_*/,
+MagickPPExport int Magick::operator == ( const Magick::PathCurvetoArgs& /*left_*/,
                                         const Magick::PathCurvetoArgs& /*right_*/ )
 {
   return ( 1 );
 }
-MagickDLLDecl int Magick::operator != ( const Magick::PathCurvetoArgs& /*left_*/,
+MagickPPExport int Magick::operator != ( const Magick::PathCurvetoArgs& /*left_*/,
                                         const Magick::PathCurvetoArgs& /*right_*/ )
 {
   return ( 0 );
 }
-MagickDLLDecl int Magick::operator > ( const Magick::PathCurvetoArgs& /*left_*/,
+MagickPPExport int Magick::operator > ( const Magick::PathCurvetoArgs& /*left_*/,
                                        const Magick::PathCurvetoArgs& /*right_*/ )
 {
   return ( 0 );
 }
-MagickDLLDecl int Magick::operator <  ( const Magick::PathCurvetoArgs& /*left_*/,
+MagickPPExport int Magick::operator <  ( const Magick::PathCurvetoArgs& /*left_*/,
                                         const Magick::PathCurvetoArgs& /*right_*/ )
 {
   return  ( false );
 }
-MagickDLLDecl int Magick::operator >= ( const Magick::PathCurvetoArgs& left_,
+MagickPPExport int Magick::operator >= ( const Magick::PathCurvetoArgs& left_,
                                         const Magick::PathCurvetoArgs& right_ )
 {
   return ( ( left_ > right_ ) || ( left_ == right_ ) );
 }
-MagickDLLDecl int Magick::operator <= ( const Magick::PathCurvetoArgs& left_,
+MagickPPExport int Magick::operator <= ( const Magick::PathCurvetoArgs& left_,
                                         const Magick::PathCurvetoArgs& right_ )
 {
   return ( ( left_ < right_ ) || ( left_ == right_ ) );
@@ -1709,8 +2063,9 @@ void Magick::PathSmoothCurvetoAbs::operator()
       double x2 = p->x();
       double y2 = p->y();
       p++;
-      if(p != _coordinates.end() )
-        DrawPathCurveToSmoothAbsolute( context_, x2, y2, p->x(), p->y() );
+      if (p == _coordinates.end() )
+        break;
+      DrawPathCurveToSmoothAbsolute( context_, x2, y2, p->x(), p->y() );
     }
 }
 Magick::VPathBase* Magick::PathSmoothCurvetoAbs::copy() const
@@ -1745,8 +2100,9 @@ void Magick::PathSmoothCurvetoRel::operator()
       double x2 = p->x();
       double y2 = p->y();
       p++;
-      if(p != _coordinates.end() )
-        DrawPathCurveToSmoothRelative( context_, x2, y2, p->x(), p->y() );
+      if (p == _coordinates.end() )
+        break;
+      DrawPathCurveToSmoothRelative( context_, x2, y2, p->x(), p->y() );
     }
 }
 Magick::VPathBase* Magick::PathSmoothCurvetoRel::copy() const
@@ -1757,37 +2113,37 @@ Magick::VPathBase* Magick::PathSmoothCurvetoRel::copy() const
 //
 // Quadratic Curveto (Quadratic Bezier)
 //
-MagickDLLDecl int Magick::operator ==
+MagickPPExport int Magick::operator ==
 ( const Magick::PathQuadraticCurvetoArgs& /*left_*/,
   const Magick::PathQuadraticCurvetoArgs& /*right_*/ )
 {
   return ( 1 );
 }
-MagickDLLDecl int Magick::operator !=
+MagickPPExport int Magick::operator !=
 ( const Magick::PathQuadraticCurvetoArgs& /*left_*/,
   const Magick::PathQuadraticCurvetoArgs& /*right_*/ )
 {
   return ( 0 );
 }
-MagickDLLDecl int Magick::operator >
+MagickPPExport int Magick::operator >
 ( const Magick::PathQuadraticCurvetoArgs& /*left_*/,
   const Magick::PathQuadraticCurvetoArgs& /*right_*/ )
 {
   return ( 0 );
 }
-MagickDLLDecl int Magick::operator < 
+MagickPPExport int Magick::operator < 
 ( const Magick::PathQuadraticCurvetoArgs& /*left_*/,
   const Magick::PathQuadraticCurvetoArgs& /*right_*/ )
 {
   return  ( 0 );
 }
-MagickDLLDecl int Magick::operator >=
+MagickPPExport int Magick::operator >=
 ( const Magick::PathQuadraticCurvetoArgs& left_,
   const Magick::PathQuadraticCurvetoArgs& right_ )
 {
   return ( ( left_ > right_ ) || ( left_ == right_ ) );
 }
-MagickDLLDecl int Magick::operator <=
+MagickPPExport int Magick::operator <=
 ( const Magick::PathQuadraticCurvetoArgs& left_,
   const Magick::PathQuadraticCurvetoArgs& right_ )
 {
@@ -2135,10 +2491,10 @@ Magick::VPathBase* Magick::PathMovetoRel::copy() const
 }
 
 #if defined(EXPLICIT_TEMPLATE_INSTANTIATION)
-// template class std::list<Magick::Coordinate>;
-// template class std::list<const Magick::Drawable>;
-// template class std::list<const Magick::PathArcArgs>;
-// template class std::list<const Magick::PathCurvetoArgs>;
-// template class std::list<const Magick::PathQuadraticCurvetoArgs>;
-// template class std::list<const Magick::VPath>;
+// template class std::vector<Magick::Coordinate>;
+// template class std::vector<const Magick::Drawable>;
+// template class std::vector<const Magick::PathArcArgs>;
+// template class std::vector<const Magick::PathCurvetoArgs>;
+// template class std::vector<const Magick::PathQuadraticCurvetoArgs>;
+// template class std::vector<const Magick::VPath>;
 #endif
