@@ -13,17 +13,17 @@
 %                 Read/Write Magick Vector Graphics Metafiles.                %
 %                                                                             %
 %                              Software Design                                %
-%                                John Cristy                                  %
+%                                   Cristy                                    %
 %                                 April 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -39,29 +39,29 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/artifact.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/draw.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/module.h"
-#include "magick/property.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/string_.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/artifact.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/draw.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/module.h"
+#include "MagickCore/property.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/string_.h"
 
 /*
   Forward declarations.
 */
 static MagickBooleanType
-  WriteMVGImage(const ImageInfo *,Image *);
+  WriteMVGImage(const ImageInfo *,Image *,ExceptionInfo *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -83,18 +83,16 @@ static MagickBooleanType
 %
 %  A description of each parameter follows:
 %
-%    o magick: This string is generally the first few bytes of an image file
-%      or blob.
+%    o magick: compare image format pattern against these bytes.
 %
 %    o length: Specifies the length of the magick string.
 %
-%
 */
 static MagickBooleanType IsMVG(const unsigned char *magick,const size_t length)
 {
   if (length < 20)
     return(MagickFalse);
-  if (LocaleNCompare((char *) magick,"push graphic-context",20) == 0)
+  if (LocaleNCompare((const char *) magick,"push graphic-context",20) == 0)
     return(MagickTrue);
   return(MagickFalse);
 }
@@ -128,8 +126,6 @@ static MagickBooleanType IsMVG(const unsigned char *magick,const size_t length)
 */
 static Image *ReadMVGImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
-#define BoundingBox  "viewbox"
-
   DrawInfo
     *draw_info;
 
@@ -139,20 +135,17 @@ static Image *ReadMVGImage(const ImageInfo *image_info,ExceptionInfo *exception)
   MagickBooleanType
     status;
 
-  unsigned char
-    *primitive;
-
   /*
     Open image.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  image=AcquireImage(image_info);
+  assert(exception->signature == MagickCoreSignature);
+  image=AcquireImage(image_info,exception);
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
@@ -162,7 +155,7 @@ static Image *ReadMVGImage(const ImageInfo *image_info,ExceptionInfo *exception)
   if ((image->columns == 0) || (image->rows == 0))
     {
       char
-        primitive[MaxTextExtent];
+        primitive[MagickPathExtent];
 
       register char
         *p;
@@ -173,44 +166,61 @@ static Image *ReadMVGImage(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Determine size of image canvas.
       */
+      (void) memset(&bounds,0,sizeof(bounds));
       while (ReadBlobString(image,primitive) != (char *) NULL)
       {
+        int
+          count;
+
         for (p=primitive; (*p == ' ') || (*p == '\t'); p++) ;
-        if (LocaleNCompare(BoundingBox,p,strlen(BoundingBox)) != 0)
-          continue;
-        (void) sscanf(p,"viewbox %lf %lf %lf %lf",&bounds.x1,&bounds.y1,
+        count=sscanf(p,"viewbox %lf %lf %lf %lf",&bounds.x1,&bounds.y1,
           &bounds.x2,&bounds.y2);
-        image->columns=(unsigned long) ((bounds.x2-bounds.x1)+0.5);
-        image->rows=(unsigned long) ((bounds.y2-bounds.y1)+0.5);
+        if (count != 4)
+          continue;
+        image->columns=(size_t) floor((bounds.x2-bounds.x1)+0.5);
+        image->rows=(size_t) floor((bounds.y2-bounds.y1)+0.5);
         break;
       }
     }
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(OptionError,"MustSpecifyImageSize");
   draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
-  draw_info->affine.sx=image->x_resolution == 0.0 ? 1.0 : image->x_resolution/
-    DefaultResolution;
-  draw_info->affine.sy=image->y_resolution == 0.0 ? 1.0 : image->y_resolution/
-    DefaultResolution;
-  image->columns=(unsigned long) (draw_info->affine.sx*image->columns);
-  image->rows=(unsigned long) (draw_info->affine.sy*image->rows);
-  if (SetImageExtent(image,0,0) == MagickFalse)
+  draw_info->affine.sx=image->resolution.x == 0.0 ? 1.0 : image->resolution.x/
+    96.0;
+  draw_info->affine.sy=image->resolution.y == 0.0 ? 1.0 : image->resolution.y/
+    96.0;
+  image->columns=(size_t) (draw_info->affine.sx*image->columns);
+  image->rows=(size_t) (draw_info->affine.sy*image->rows);
+  status=SetImageExtent(image,image->columns,image->rows,exception);
+  if (status == MagickFalse)
     {
-      InheritException(exception,&image->exception);
+      draw_info=DestroyDrawInfo(draw_info);
+      return(DestroyImageList(image));
+    }
+  if (SetImageBackgroundColor(image,exception) == MagickFalse)
+    {
+      draw_info=DestroyDrawInfo(draw_info);
       return(DestroyImageList(image));
     }
-  (void) SetImageBackgroundColor(image);
   /*
     Render drawing.
   */
-  primitive=GetBlobStreamData(image);
-  if (primitive != (unsigned char *) NULL)
-    draw_info->primitive=AcquireString((char *) primitive);
-  else
+  if (GetBlobStreamData(image) == (unsigned char *) NULL)
     draw_info->primitive=FileToString(image->filename,~0UL,exception);
+  else
+    {
+      draw_info->primitive=(char *) AcquireMagickMemory(GetBlobSize(image)+1);
+      if (draw_info->primitive != (char *) NULL)
+        {
+          memcpy(draw_info->primitive,GetBlobStreamData(image),
+            GetBlobSize(image));
+          draw_info->primitive[GetBlobSize(image)]='\0';
+        }
+     }
   if (draw_info->primitive == (char *) NULL)
-    return((Image *) NULL);
-  (void) DrawImage(image,draw_info);
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  (void) DrawImage(image,draw_info,exception);
+  (void) SetImageArtifact(image,"MVG",draw_info->primitive);
   draw_info=DestroyDrawInfo(draw_info);
   (void) CloseBlob(image);
   return(GetFirstImageInList(image));
@@ -236,22 +246,20 @@ static Image *ReadMVGImage(const ImageInfo *image_info,ExceptionInfo *exception)
 %
 %  The format of the RegisterMVGImage method is:
 %
-%      unsigned long RegisterMVGImage(void)
+%      size_t RegisterMVGImage(void)
 %
 */
-ModuleExport unsigned long RegisterMVGImage(void)
+ModuleExport size_t RegisterMVGImage(void)
 {
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("MVG");
+  entry=AcquireMagickInfo("MVG","MVG","Magick Vector Graphics");
   entry->decoder=(DecodeImageHandler *) ReadMVGImage;
   entry->encoder=(EncodeImageHandler *) WriteMVGImage;
   entry->magick=(IsImageFormatHandler *) IsMVG;
-  entry->adjoin=MagickFalse;
-  entry->seekable_stream=MagickTrue;
-  entry->description=ConstantString("Magick Vector Graphics");
-  entry->module=ConstantString("MVG");
+  entry->format_type=ImplicitFormatType;
+  entry->flags^=CoderAdjoinFlag;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -295,7 +303,8 @@ ModuleExport void UnregisterMVGImage(void)
 %
 %  The format of the WriteMVGImage method is:
 %
-%      MagickBooleanType WriteMVGImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteMVGImage(const ImageInfo *image_info,
+%        Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -303,8 +312,11 @@ ModuleExport void UnregisterMVGImage(void)
 %
 %    o image:  The image.
 %
+%    o exception: return any errors or warnings in this structure.
+%
 */
-static MagickBooleanType WriteMVGImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteMVGImage(const ImageInfo *image_info,Image *image,
+  ExceptionInfo *exception)
 {
   const char
     *value;
@@ -316,18 +328,18 @@ static MagickBooleanType WriteMVGImage(const ImageInfo *image_info,Image *image)
     Open output image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   value=GetImageArtifact(image,"MVG");
   if (value == (const char *) NULL)
     ThrowWriterException(OptionError,"NoImageVectorGraphics");
-  status=OpenBlob(image_info,image,WriteBlobMode,&image->exception);
+  status=OpenBlob(image_info,image,WriteBlobMode,exception);
   if (status == MagickFalse)
     return(status);
-  (void) WriteBlob(image,strlen(value),(unsigned char *) value);
+  (void) WriteBlob(image,strlen(value),(const unsigned char *) value);
   (void) CloseBlob(image);
   return(MagickTrue);
 }
