@@ -10,20 +10,20 @@
 %                            R  R   A   A  W   W                              %
 %                                                                             %
 %                                                                             %
-%                       Read/Write RAW Image Format.                          %
+%                       Read/Write RAW Image Format                           %
 %                                                                             %
 %                              Software Design                                %
-%                                John Cristy                                  %
+%                                   Cristy                                    %
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2008 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2018 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
 %  obtain a copy of the License at                                            %
 %                                                                             %
-%    http://www.imagemagick.org/script/license.php                            %
+%    https://www.imagemagick.org/script/license.php                           %
 %                                                                             %
 %  Unless required by applicable law or agreed to in writing, software        %
 %  distributed under the License is distributed on an "AS IS" BASIS,          %
@@ -39,32 +39,34 @@
 /*
   Include declarations.
 */
-#include "magick/studio.h"
-#include "magick/blob.h"
-#include "magick/blob-private.h"
-#include "magick/colorspace.h"
-#include "magick/constitute.h"
-#include "magick/exception.h"
-#include "magick/exception-private.h"
-#include "magick/image.h"
-#include "magick/image-private.h"
-#include "magick/list.h"
-#include "magick/magick.h"
-#include "magick/memory_.h"
-#include "magick/monitor.h"
-#include "magick/monitor-private.h"
-#include "magick/quantum-private.h"
-#include "magick/quantum-private.h"
-#include "magick/static.h"
-#include "magick/statistic.h"
-#include "magick/string_.h"
-#include "magick/module.h"
+#include "MagickCore/studio.h"
+#include "MagickCore/blob.h"
+#include "MagickCore/blob-private.h"
+#include "MagickCore/cache.h"
+#include "MagickCore/colorspace.h"
+#include "MagickCore/constitute.h"
+#include "MagickCore/exception.h"
+#include "MagickCore/exception-private.h"
+#include "MagickCore/image.h"
+#include "MagickCore/image-private.h"
+#include "MagickCore/list.h"
+#include "MagickCore/magick.h"
+#include "MagickCore/memory_.h"
+#include "MagickCore/monitor.h"
+#include "MagickCore/monitor-private.h"
+#include "MagickCore/pixel-accessor.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/quantum-private.h"
+#include "MagickCore/static.h"
+#include "MagickCore/statistic.h"
+#include "MagickCore/string_.h"
+#include "MagickCore/module.h"
 
 /*
   Forward declarations.
 */
 static MagickBooleanType
-  WriteRAWImage(const ImageInfo *,Image *);
+  WriteRAWImage(const ImageInfo *,Image *,ExceptionInfo *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -92,85 +94,76 @@ static MagickBooleanType
 %    o exception: return any errors or warnings in this structure.
 %
 */
-static Image *ReadRAWImage(const ImageInfo *image_info,
-  ExceptionInfo *exception)
+static Image *ReadRAWImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
+  const unsigned char
+    *pixels;
+
   Image
     *canvas_image,
     *image;
 
-  long
-    y;
-
   MagickBooleanType
     status;
 
+  MagickOffsetType
+    scene;
+
   QuantumInfo
     *quantum_info;
 
   QuantumType
     quantum_type;
 
-  register const IndexPacket
-    *canvas_indexes;
-
-  register const PixelPacket
-    *p;
-
-  register long
-    i,
-    x;
-
-  register IndexPacket
-    *indexes;
-
-  register PixelPacket
-    *q;
-
-  ssize_t
-    count;
-
   size_t
     length;
 
-  unsigned char
-    *pixels;
+  ssize_t
+    count,
+    y;
 
   /*
     Open image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   if (image_info->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  assert(exception->signature == MagickSignature);
-  image=AcquireImage(image_info);
+  assert(exception->signature == MagickCoreSignature);
+  image=AcquireImage(image_info,exception);
   if ((image->columns == 0) || (image->rows == 0))
     ThrowReaderException(OptionError,"MustSpecifyImageSize");
+  status=SetImageExtent(image,image->columns,image->rows,exception);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
   if (status == MagickFalse)
     {
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
-  for (i=0; i < image->offset; i++)
-    if (ReadBlobByte(image) == EOF)
-      {
-        ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
-          image->filename);
-        break;
-      }
+  if (DiscardBlobBytes(image,(MagickSizeType) image->offset) == MagickFalse)
+    ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+      image->filename);
   /*
     Create virtual canvas to support cropping (i.e. image.gray[100x100+10+20]).
   */
-  canvas_image=CloneImage(image,image->extract_info.width,1,MagickTrue,
+  canvas_image=CloneImage(image,image->extract_info.width,1,MagickFalse,
+    exception);
+  if (canvas_image == (Image *) NULL)
+    ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+  (void) SetImageVirtualPixelMethod(canvas_image,BlackVirtualPixelMethod,
     exception);
-  (void) SetImageVirtualPixelMethod(canvas_image,BlackVirtualPixelMethod);
   quantum_type=GrayQuantum;
   quantum_info=AcquireQuantumInfo(image_info,canvas_image);
-  pixels=GetQuantumPixels(quantum_info);
+  if (quantum_info == (QuantumInfo *) NULL)
+    {
+      canvas_image=DestroyImage(canvas_image);
+      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+    }
+  pixels=(const unsigned char *) NULL;
   if (image_info->number_scenes != 0)
     while (image->scene < image_info->scene)
     {
@@ -179,13 +172,18 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
       */
       image->scene++;
       length=GetQuantumExtent(canvas_image,quantum_info,quantum_type);
-      for (y=0; y < (long) image->rows; y++)
+      for (y=0; y < (ssize_t) image->rows; y++)
       {
-        count=ReadBlob(image,length,pixels);
+        pixels=(const unsigned char *) ReadBlobStream(image,length,
+          GetQuantumPixels(quantum_info),&count);
         if (count != (ssize_t) length)
           break;
       }
     }
+  scene=0;
+  count=0;
+  length=0;
+  status=MagickTrue;
   do
   {
     /*
@@ -194,49 +192,73 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
     if ((image_info->ping != MagickFalse) && (image_info->number_scenes != 0))
       if (image->scene >= (image_info->scene+image_info->number_scenes-1))
         break;
-    if ((SetImageExtent(image,0,0) == MagickFalse) ||
-        (SetImageExtent(canvas_image,0,0) == MagickFalse))
+    status=SetImageExtent(image,image->columns,image->rows,exception);
+    if (status == MagickFalse)
       break;
-    length=GetQuantumExtent(canvas_image,quantum_info,GrayQuantum);
-    count=ReadBlob(image,length,pixels);
-    if (count != (ssize_t) length)
-      break;
-    for (y=0; y < (long) image->extract_info.height; y++)
+    if (scene == 0)
+      {
+        length=GetQuantumExtent(canvas_image,quantum_info,quantum_type);
+        pixels=(const unsigned char *) ReadBlobStream(image,length,
+          GetQuantumPixels(quantum_info),&count);
+        if (count != (ssize_t) length)
+          break;
+      }
+    for (y=0; y < (ssize_t) image->extract_info.height; y++)
     {
-      q=GetImagePixels(canvas_image,0,0,canvas_image->columns,1);
-      if (q == (PixelPacket *) NULL)
-        break;
-      length=ImportQuantumPixels(canvas_image,quantum_info,quantum_type,pixels);
-      if (SyncImagePixels(canvas_image) == MagickFalse)
-        break;
-      count=ReadBlob(image,length,pixels);
-      if (((y-image->extract_info.y) >= 0) && 
-          ((y-image->extract_info.y) < (long) image->rows))
+      register const Quantum
+        *magick_restrict p;
+
+      register Quantum
+        *magick_restrict q;
+
+      register ssize_t
+        x;
+
+      if (count != (ssize_t) length)
         {
-          p=AcquireImagePixels(canvas_image,canvas_image->extract_info.x,0,
-            canvas_image->columns,1,exception);
-          q=SetImagePixels(image,0,y-image->extract_info.y,image->columns,1);
-          if ((p == (const PixelPacket *) NULL) || (q == (PixelPacket *) NULL))
+          status=MagickFalse;
+          ThrowFileException(exception,CorruptImageError,"UnexpectedEndOfFile",
+            image->filename);
+          break;
+        }
+      q=GetAuthenticPixels(canvas_image,0,0,canvas_image->columns,1,exception);
+      if (q == (Quantum *) NULL)
+        break;
+      length=ImportQuantumPixels(canvas_image,(CacheView *) NULL,quantum_info,
+        quantum_type,pixels,exception);
+      if (SyncAuthenticPixels(canvas_image,exception) == MagickFalse)
+        break;
+      if (((y-image->extract_info.y) >= 0) && 
+          ((y-image->extract_info.y) < (ssize_t) image->rows))
+        {
+          p=GetVirtualPixels(canvas_image,canvas_image->extract_info.x,0,
+            image->columns,1,exception);
+          q=QueueAuthenticPixels(image,0,y-image->extract_info.y,image->columns,
+            1,exception);
+          if ((p == (const Quantum *) NULL) || (q == (Quantum *) NULL))
             break;
-          canvas_indexes=AcquireIndexes(canvas_image);
-          indexes=GetIndexes(image);
-          for (x=0; x < (long) image->columns; x++)
+          for (x=0; x < (ssize_t) image->columns; x++)
           {
-            q->red=p->red;
-            q->green=p->green;
-            q->blue=p->blue;
-            p++;
-            q++;
+            SetPixelRed(image,GetPixelRed(canvas_image,p),q);
+            SetPixelGreen(image,GetPixelGreen(canvas_image,p),q);
+            SetPixelBlue(image,GetPixelBlue(canvas_image,p),q);
+            p+=GetPixelChannels(canvas_image);
+            q+=GetPixelChannels(image);
           }
-          if (SyncImagePixels(image) == MagickFalse)
+          if (SyncAuthenticPixels(image,exception) == MagickFalse)
             break;
         }
       if (image->previous == (Image *) NULL)
         {
-          status=SetImageProgress(image,LoadImageTag,y,image->rows);
+          status=SetImageProgress(image,LoadImageTag,(MagickOffsetType) y,
+            image->rows);
           if (status == MagickFalse)
             break;
         }
+      pixels=(const unsigned char *) ReadBlobStream(image,length,
+        GetQuantumPixels(quantum_info),&count);
+      if (count != (ssize_t) length)
+        break;
     }
     SetQuantumImageType(image,quantum_type);
     /*
@@ -250,7 +272,7 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
         /*
           Allocate next image structure.
         */
-        AcquireNextImage(image_info,image);
+        AcquireNextImage(image_info,image,exception);
         if (GetNextImageInList(image) == (Image *) NULL)
           {
             image=DestroyImageList(image);
@@ -262,11 +284,13 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
         if (status == MagickFalse)
           break;
       }
+    scene++;
   } while (count == (ssize_t) length);
   quantum_info=DestroyQuantumInfo(quantum_info);
-  InheritException(exception,&canvas_image->exception);
   canvas_image=DestroyImage(canvas_image);
   (void) CloseBlob(image);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
   return(GetFirstImageInList(image));
 }
 
@@ -289,85 +313,76 @@ static Image *ReadRAWImage(const ImageInfo *image_info,
 %
 %  The format of the RegisterRAWImage method is:
 %
-%      unsigned long RegisterRAWImage(void)
+%      size_t RegisterRAWImage(void)
 %
 */
-ModuleExport unsigned long RegisterRAWImage(void)
+ModuleExport size_t RegisterRAWImage(void)
 {
   MagickInfo
     *entry;
 
-  entry=SetMagickInfo("R");
+  entry=AcquireMagickInfo("RAW","R","Raw red samples");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
   entry->encoder=(EncodeImageHandler *) WriteRAWImage;
-  entry->raw=MagickTrue;
-  entry->endian_support=MagickTrue;
-  entry->description=ConstantString("Raw red samples");
-  entry->module=ConstantString("RAW");
+  entry->flags|=CoderRawSupportFlag;
+  entry->flags|=CoderEndianSupportFlag;
+  entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("C");
+  entry=AcquireMagickInfo("RAW","C","Raw cyan samples");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
   entry->encoder=(EncodeImageHandler *) WriteRAWImage;
-  entry->raw=MagickTrue;
-  entry->endian_support=MagickTrue;
-  entry->description=ConstantString("Raw cyan samples");
-  entry->module=ConstantString("RAW");
+  entry->flags|=CoderRawSupportFlag;
+  entry->flags|=CoderEndianSupportFlag;
+  entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("G");
+  entry=AcquireMagickInfo("RAW","G","Raw green samples");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
   entry->encoder=(EncodeImageHandler *) WriteRAWImage;
-  entry->raw=MagickTrue;
-  entry->endian_support=MagickTrue;
-  entry->description=ConstantString("Raw green samples");
-  entry->module=ConstantString("RAW");
+  entry->flags|=CoderRawSupportFlag;
+  entry->flags|=CoderEndianSupportFlag;
+  entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("M");
+  entry=AcquireMagickInfo("RAW","M","Raw magenta samples");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
   entry->encoder=(EncodeImageHandler *) WriteRAWImage;
-  entry->raw=MagickTrue;
-  entry->endian_support=MagickTrue;
-  entry->description=ConstantString("Raw magenta samples");
-  entry->module=ConstantString("RAW");
+  entry->flags|=CoderRawSupportFlag;
+  entry->flags|=CoderEndianSupportFlag;
+  entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("B");
+  entry=AcquireMagickInfo("RAW","B","Raw blue samples");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
   entry->encoder=(EncodeImageHandler *) WriteRAWImage;
-  entry->raw=MagickTrue;
-  entry->endian_support=MagickTrue;
-  entry->description=ConstantString("Raw blue samples");
-  entry->module=ConstantString("RAW");
+  entry->flags|=CoderRawSupportFlag;
+  entry->flags|=CoderEndianSupportFlag;
+  entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("Y");
+  entry=AcquireMagickInfo("RAW","Y","Raw yellow samples");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
   entry->encoder=(EncodeImageHandler *) WriteRAWImage;
-  entry->raw=MagickTrue;
-  entry->endian_support=MagickTrue;
-  entry->description=ConstantString("Raw yellow samples");
-  entry->module=ConstantString("RAW");
+  entry->flags|=CoderRawSupportFlag;
+  entry->flags|=CoderEndianSupportFlag;
+  entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("A");
+  entry=AcquireMagickInfo("RAW","A","Raw alpha samples");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
   entry->encoder=(EncodeImageHandler *) WriteRAWImage;
-  entry->raw=MagickTrue;
-  entry->endian_support=MagickTrue;
-  entry->description=ConstantString("Raw alpha samples");
-  entry->module=ConstantString("RAW");
+  entry->flags|=CoderRawSupportFlag;
+  entry->flags|=CoderEndianSupportFlag;
+  entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("O");
+  entry=AcquireMagickInfo("RAW","O","Raw opacity samples");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
   entry->encoder=(EncodeImageHandler *) WriteRAWImage;
-  entry->raw=MagickTrue;
-  entry->endian_support=MagickTrue;
-  entry->description=ConstantString("Raw opacity samples");
-  entry->module=ConstantString("RAW");
+  entry->flags|=CoderRawSupportFlag;
+  entry->flags|=CoderEndianSupportFlag;
+  entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
-  entry=SetMagickInfo("K");
+  entry=AcquireMagickInfo("RAW","K","Raw black samples");
   entry->decoder=(DecodeImageHandler *) ReadRAWImage;
   entry->encoder=(EncodeImageHandler *) WriteRAWImage;
-  entry->raw=MagickTrue;
-  entry->endian_support=MagickTrue;
-  entry->description=ConstantString("Raw black samples");
-  entry->module=ConstantString("RAW");
+  entry->flags|=CoderRawSupportFlag;
+  entry->flags|=CoderEndianSupportFlag;
+  entry->format_type=ImplicitFormatType;
   (void) RegisterMagickInfo(entry);
   return(MagickImageCoderSignature);
 }
@@ -419,7 +434,8 @@ ModuleExport void UnregisterRAWImage(void)
 %
 %  The format of the WriteRAWImage method is:
 %
-%      MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
+%      MagickBooleanType WriteRAWImage(const ImageInfo *image_info,
+%        Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows.
 %
@@ -427,13 +443,12 @@ ModuleExport void UnregisterRAWImage(void)
 %
 %    o image:  The image.
 %
+%    o exception: return any errors or warnings in this structure.
 %
 */
-static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
+static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image,
+  ExceptionInfo *exception)
 {
-  long
-    y;
-
   MagickOffsetType
     scene;
 
@@ -446,15 +461,17 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
   MagickBooleanType
     status;
 
-  register const PixelPacket
+  register const Quantum
     *p;
 
-  ssize_t
-    count;
-
   size_t
+    imageListLength,
     length;
 
+  ssize_t
+    count,
+    y;
+
   unsigned char
     *pixels;
 
@@ -462,12 +479,14 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
     Open output image file.
   */
   assert(image_info != (const ImageInfo *) NULL);
-  assert(image_info->signature == MagickSignature);
+  assert(image_info->signature == MagickCoreSignature);
   assert(image != (Image *) NULL);
-  assert(image->signature == MagickSignature);
+  assert(image->signature == MagickCoreSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
+  assert(exception != (ExceptionInfo *) NULL);
+  assert(exception->signature == MagickCoreSignature);
+  status=OpenBlob(image_info,image,WriteBinaryBlobMode,exception);
   if (status == MagickFalse)
     return(status);
   switch (*image->magick)
@@ -547,25 +566,30 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
     }
   }
   scene=0;
+  imageListLength=GetImageListLength(image);
   do
   {
     /*
       Convert image to RAW raster pixels.
     */
     quantum_info=AcquireQuantumInfo(image_info,image);
-    pixels=GetQuantumPixels(quantum_info);
-    for (y=0; y < (long) image->rows; y++)
+    if (quantum_info == (QuantumInfo *) NULL)
+      ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
+    pixels=(unsigned char *) GetQuantumPixels(quantum_info);
+    for (y=0; y < (ssize_t) image->rows; y++)
     {
-      p=AcquireImagePixels(image,0,y,image->columns,1,&image->exception);
-      if (p == (const PixelPacket *) NULL)
+      p=GetVirtualPixels(image,0,y,image->columns,1,exception);
+      if (p == (const Quantum *) NULL)
         break;
-      length=ExportQuantumPixels(image,quantum_info,quantum_type,pixels);
+      length=ExportQuantumPixels(image,(CacheView *) NULL,quantum_info,
+        quantum_type,pixels,exception);
       count=WriteBlob(image,length,pixels);
       if (count != (ssize_t) length)
         break;
       if (image->previous == (Image *) NULL)
         {
-          status=SetImageProgress(image,SaveImageTag,y,image->rows);
+          status=SetImageProgress(image,SaveImageTag,(MagickOffsetType) y,
+            image->rows);
           if (status == MagickFalse)
             break;
         }
@@ -574,8 +598,7 @@ static MagickBooleanType WriteRAWImage(const ImageInfo *image_info,Image *image)
     if (GetNextImageInList(image) == (Image *) NULL)
       break;
     image=SyncNextImageInList(image);
-    status=SetImageProgress(image,SaveImagesTag,scene++,
-      GetImageListLength(image));
+    status=SetImageProgress(image,SaveImagesTag,scene++,imageListLength);
     if (status == MagickFalse)
       break;
   } while (image_info->adjoin != MagickFalse);
